@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -18,7 +18,7 @@
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
 
-#include <boost/pointer_to_other.hpp>
+#include <boost/intrusive/pointer_traits.hpp>
 
 #include <boost/interprocess/interprocess_fwd.hpp>
 #include <boost/assert.hpp>
@@ -33,7 +33,7 @@
 #include <cstddef>
 
 //!\file
-//!Describes private_adaptive_pool_base pooled shared memory STL compatible allocator 
+//!Describes private_adaptive_pool_base pooled shared memory STL compatible allocator
 
 namespace boost {
 namespace interprocess {
@@ -81,10 +81,12 @@ class private_adaptive_pool_base
    /// @endcond
 
    public:
-   typedef typename boost::
-      pointer_to_other<void_pointer, T>::type            pointer;
-   typedef typename boost::
-      pointer_to_other<void_pointer, const T>::type      const_pointer;
+   typedef typename boost::intrusive::
+      pointer_traits<void_pointer>::template
+         rebind_pointer<T>::type                         pointer;
+   typedef typename boost::intrusive::
+      pointer_traits<void_pointer>::template
+         rebind_pointer<const T>::type                   const_pointer;
    typedef T                                             value_type;
    typedef typename ipcdetail::add_reference
                      <value_type>::type                  reference;
@@ -94,13 +96,13 @@ class private_adaptive_pool_base
    typedef typename segment_manager::size_type           difference_type;
    typedef boost::interprocess::version_type
       <private_adaptive_pool_base, Version>              version;
-   typedef boost::container::containers_detail::transform_multiallocation_chain
+   typedef boost::container::container_detail::transform_multiallocation_chain
       <typename SegmentManager::multiallocation_chain, T>multiallocation_chain;
 
    //!Obtains node_allocator from other node_allocator
    template<class T2>
    struct rebind
-   {  
+   {
       typedef private_adaptive_pool_base
          <Version, T2, SegmentManager, NodesPerBlock, MaxFreeBlocks, OverheadPercent>   other;
    };
@@ -152,7 +154,7 @@ class private_adaptive_pool_base
    {}
 
    //!Destructor, frees all used memory. Never throws
-   ~private_adaptive_pool_base() 
+   ~private_adaptive_pool_base()
    {}
 
    //!Returns the segment manager. Never throws
@@ -176,13 +178,13 @@ class private_adaptive_pool_base
 
 //!Equality test for same type of private_adaptive_pool_base
 template<unsigned int V, class T, class S, std::size_t NodesPerBlock, std::size_t F, unsigned char OP> inline
-bool operator==(const private_adaptive_pool_base<V, T, S, NodesPerBlock, F, OP> &alloc1, 
+bool operator==(const private_adaptive_pool_base<V, T, S, NodesPerBlock, F, OP> &alloc1,
                 const private_adaptive_pool_base<V, T, S, NodesPerBlock, F, OP> &alloc2)
 {  return &alloc1 == &alloc2; }
 
 //!Inequality test for same type of private_adaptive_pool_base
 template<unsigned int V, class T, class S, std::size_t NodesPerBlock, std::size_t F, unsigned char OP> inline
-bool operator!=(const private_adaptive_pool_base<V, T, S, NodesPerBlock, F, OP> &alloc1, 
+bool operator!=(const private_adaptive_pool_base<V, T, S, NodesPerBlock, F, OP> &alloc1,
                 const private_adaptive_pool_base<V, T, S, NodesPerBlock, F, OP> &alloc2)
 {  return &alloc1 != &alloc2; }
 
@@ -208,11 +210,11 @@ class private_adaptive_pool_v1
 
    template<class T2>
    struct rebind
-   {  
+   {
       typedef private_adaptive_pool_v1<T2, SegmentManager, NodesPerBlock, MaxFreeBlocks, OverheadPercent>  other;
    };
 
-   private_adaptive_pool_v1(SegmentManager *segment_mngr) 
+   private_adaptive_pool_v1(SegmentManager *segment_mngr)
       : base_t(segment_mngr)
    {}
 
@@ -227,7 +229,7 @@ class private_adaptive_pool_v1
 
 /// @endcond
 
-//!An STL node allocator that uses a segment manager as memory 
+//!An STL node allocator that uses a segment manager as memory
 //!source. The internal pointer type will of the same type (raw, smart) as
 //!"typename SegmentManager::void_pointer" type. This allows
 //!placing the allocator in shared memory, memory mapped-files, etc...
@@ -267,12 +269,12 @@ class private_adaptive_pool
 
    template<class T2>
    struct rebind
-   {  
+   {
       typedef private_adaptive_pool
          <T2, SegmentManager, NodesPerBlock, MaxFreeBlocks, OverheadPercent>  other;
    };
 
-   private_adaptive_pool(SegmentManager *segment_mngr) 
+   private_adaptive_pool(SegmentManager *segment_mngr)
       : base_t(segment_mngr)
    {}
 
@@ -296,11 +298,11 @@ class private_adaptive_pool
    typedef typename segment_manager::size_type           size_type;
    typedef typename segment_manager::difference_type     difference_type;
 
-   //!Obtains private_adaptive_pool from 
+   //!Obtains private_adaptive_pool from
    //!private_adaptive_pool
    template<class T2>
    struct rebind
-   {  
+   {
       typedef private_adaptive_pool
          <T2, SegmentManager, NodesPerBlock, MaxFreeBlocks, OverheadPercent> other;
    };
@@ -312,7 +314,7 @@ class private_adaptive_pool
    private_adaptive_pool& operator=
       (const private_adaptive_pool<T2, SegmentManager2, N2, F2>&);
 
-   //!Not assignable from 
+   //!Not assignable from
    //!other private_adaptive_pool
    private_adaptive_pool& operator=(const private_adaptive_pool&);
 
@@ -322,7 +324,7 @@ class private_adaptive_pool
    //!Can throw boost::interprocess::bad_alloc
    private_adaptive_pool(segment_manager *segment_mngr);
 
-   //!Copy constructor from other private_adaptive_pool. Increments the reference 
+   //!Copy constructor from other private_adaptive_pool. Increments the reference
    //!count of the associated node pool. Never throws
    private_adaptive_pool(const private_adaptive_pool &other);
 
@@ -349,7 +351,7 @@ class private_adaptive_pool
    //!Never throws
    size_type max_size() const;
 
-   //!Allocate memory for an array of count elements. 
+   //!Allocate memory for an array of count elements.
    //!Throws boost::interprocess::bad_alloc if there is no enough memory
    pointer allocate(size_type count, cvoid_pointer hint = 0);
 
@@ -373,7 +375,7 @@ class private_adaptive_pool
    //!Never throws
    const_pointer address(const_reference value) const;
 
-   //!Copy construct an object. 
+   //!Copy construct an object.
    //!Throws if T's copy constructor throws
    void construct(const pointer &ptr, const_reference v);
 
@@ -388,7 +390,7 @@ class private_adaptive_pool
 
    std::pair<pointer, bool>
       allocation_command(boost::interprocess::allocation_type command,
-                         size_type limit_size, 
+                         size_type limit_size,
                          size_type preferred_size,
                          size_type &received_size, const pointer &reuse = 0);
 
@@ -446,13 +448,13 @@ class private_adaptive_pool
 //!Equality test for same type
 //!of private_adaptive_pool
 template<class T, class S, std::size_t NodesPerBlock, std::size_t F, unsigned char OP> inline
-bool operator==(const private_adaptive_pool<T, S, NodesPerBlock, F, OP> &alloc1, 
+bool operator==(const private_adaptive_pool<T, S, NodesPerBlock, F, OP> &alloc1,
                 const private_adaptive_pool<T, S, NodesPerBlock, F, OP> &alloc2);
 
 //!Inequality test for same type
 //!of private_adaptive_pool
 template<class T, class S, std::size_t NodesPerBlock, std::size_t F, unsigned char OP> inline
-bool operator!=(const private_adaptive_pool<T, S, NodesPerBlock, F, OP> &alloc1, 
+bool operator!=(const private_adaptive_pool<T, S, NodesPerBlock, F, OP> &alloc1,
                 const private_adaptive_pool<T, S, NodesPerBlock, F, OP> &alloc2);
 
 #endif
