@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2011. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -24,6 +24,7 @@
 #include <boost/interprocess/sync/windows/winapi_semaphore_wrapper.hpp>
 #include <boost/interprocess/sync/windows/winapi_mutex_wrapper.hpp>
 #include <boost/unordered/unordered_map.hpp>
+#include <boost/container/map.hpp>
 #include <cstddef>
 
 namespace boost {
@@ -52,22 +53,36 @@ inline bool bytes_to_str(const void *mem, const std::size_t mem_length, char *ou
    return true;
 }
 
-struct sync_id
+class sync_id
 {
+   public:
    typedef __int64 internal_type;
-   internal_type rand;
+   sync_id(const void *map_addr)
+      : map_addr_(map_addr)
+   {  winapi::query_performance_counter(&rand_);  }
 
-   sync_id()
-   {  winapi::query_performance_counter(&rand);  }
+   explicit sync_id(internal_type val, const void *map_addr)
+      : map_addr_(map_addr)
+   {  rand_ = val;  }
 
-   explicit sync_id(internal_type val)
-   {  rand = val;  }
+   const internal_type &internal_pod() const
+   {  return rand_;  }
+
+   internal_type &internal_pod()
+   {  return rand_;  }
+
+   const void *map_addr() const
+   {  return map_addr_;  }
 
    friend std::size_t hash_value(const sync_id &m)
-   {  return boost::hash_value(m.rand);  }
+   {  return boost::hash_value(m.rand_);  }
 
    friend bool operator==(const sync_id &l, const sync_id &r)
-   {  return l.rand == r.rand;  }
+   {  return l.rand_ == r.rand_ && l.map_addr_ == r.map_addr_;  }
+
+   private:
+   internal_type rand_;
+   const void * const map_addr_;
 };
 
 class sync_handles
@@ -76,11 +91,19 @@ class sync_handles
    enum type { MUTEX, SEMAPHORE };
 
    private:
-   typedef boost::unordered_map<sync_id, void*> map_type;
+   struct address_less
+   {
+      bool operator()(sync_id const * const l, sync_id const * const r) const
+      {  return l->map_addr() <  r->map_addr(); }
+   };
+
+   typedef boost::unordered_map<sync_id, void*> umap_type;
+   typedef boost::container::map<const sync_id*, umap_type::iterator, address_less> map_type;
    static const std::size_t LengthOfGlobal = sizeof("Global\\boost.ipc")-1;
    static const std::size_t StrSize        = LengthOfGlobal + (sizeof(sync_id)*2+1);
    typedef char NameBuf[StrSize];
 
+
    void fill_name(NameBuf &name, const sync_id &id)
    {
       const char *n = "Global\\boost.ipc";
@@ -90,13 +113,12 @@ class sync_handles
          ++i;
       } while(n[i]);
       std::size_t len = sizeof(NameBuf) - LengthOfGlobal;
-      bytes_to_str(&id.rand, sizeof(id.rand), &name[LengthOfGlobal], len);
+      bytes_to_str(&id.internal_pod(), sizeof(id.internal_pod()), &name[LengthOfGlobal], len);
    }
 
-   void erase_and_throw_if_error(void *hnd_val, const sync_id &id)
+   void throw_if_error(void *hnd_val)
    {
       if(!hnd_val){
-         map_.erase(id);
          error_info err(winapi::get_last_error());
          throw interprocess_exception(err);
       }
@@ -109,8 +131,10 @@ class sync_handles
       permissions unrestricted_security;
       unrestricted_security.set_unrestricted();
       winapi_semaphore_wrapper sem_wrapper;
-      sem_wrapper.open_or_create(name, (long)initial_count, unrestricted_security);
-      erase_and_throw_if_error(sem_wrapper.handle(), id);
+      bool created;
+      sem_wrapper.open_or_create
+         (name, (long)initial_count, winapi_semaphore_wrapper::MaxCount, unrestricted_security, created);
+      throw_if_error(sem_wrapper.handle());
       return sem_wrapper.release();
    }
 
@@ -122,15 +146,18 @@ class sync_handles
       unrestricted_security.set_unrestricted();
       winapi_mutex_wrapper mtx_wrapper;
       mtx_wrapper.open_or_create(name, unrestricted_security);
-      erase_and_throw_if_error(mtx_wrapper.handle(), id);
+      throw_if_error(mtx_wrapper.handle());
       return mtx_wrapper.release();
    }
 
    public:
    void *obtain_mutex(const sync_id &id, bool *popen_created = 0)
    {
+      umap_type::value_type v(id, (void*)0);
       scoped_lock<spin_mutex> lock(mtx_);
-      void *&hnd_val = map_[id];
+      umap_type::iterator it = umap_.insert(v).first;
+      map_[&it->first] = it;
+      void *&hnd_val = it->second;
       if(!hnd_val){
          hnd_val = open_or_create_mutex(id);
          if(popen_created) *popen_created = true;
@@ -140,8 +167,11 @@ class sync_handles
 
    void *obtain_semaphore(const sync_id &id, unsigned int initial_count, bool *popen_created = 0)
    {
+      umap_type::value_type v(id, (void*)0);
       scoped_lock<spin_mutex> lock(mtx_);
-      void *&hnd_val = map_[id];
+      umap_type::iterator it = umap_.insert(v).first;
+      map_[&it->first] = it;
+      void *&hnd_val = it->second;
       if(!hnd_val){
          hnd_val = open_or_create_semaphore(id, initial_count);
          if(popen_created) *popen_created = true;
@@ -152,15 +182,35 @@ class sync_handles
    void destroy_handle(const sync_id &id)
    {
       scoped_lock<spin_mutex> lock(mtx_);
-      map_type::iterator it = map_.find(id);
-      if(it != map_.end()){
+      umap_type::iterator it = umap_.find(id);
+      umap_type::iterator itend = umap_.end();
+
+      if(it != itend){
          winapi::close_handle(it->second);
-         map_.erase(it);
+         const map_type::key_type &k = &it->first;
+         map_.erase(k);
+         umap_.erase(it);
+      }
+   }
+
+   void destroy_syncs_in_range(const void *addr, std::size_t size)
+   {
+      sync_id low_id(addr);
+      sync_id hig_id(static_cast<const char*>(addr)+size);
+      scoped_lock<spin_mutex> lock(mtx_);
+      map_type::iterator itlow(map_.lower_bound(&low_id)),
+                         ithig(map_.lower_bound(&hig_id));
+      while(itlow != ithig){
+         void *hnd = umap_[*itlow->first];
+         winapi::close_handle(hnd);
+         umap_.erase(*itlow->first);
+         itlow = map_.erase(itlow);
       }
    }
 
    private:
    spin_mutex mtx_;
+   umap_type umap_;
    map_type map_;
 };
 
