@@ -11,7 +11,7 @@
 
 //  See http://www.boost.org/libs/config for most recent version.
 //
-//  Revision $Id: config_info.cpp 78478 2012-05-15 11:57:21Z johnmaddock $
+//  Revision $Id: config_info.cpp 79537 2012-07-15 15:59:05Z marshall $
 //
 
 #include <boost/config.hpp>
@@ -982,13 +982,8 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_ADL_BARRIER);
    PRINT_MACRO(BOOST_NO_ARGUMENT_DEPENDENT_LOOKUP);
    PRINT_MACRO(BOOST_NO_ARRAY_TYPE_SPECIALIZATIONS);
-   PRINT_MACRO(BOOST_NO_AUTO_DECLARATIONS);
-   PRINT_MACRO(BOOST_NO_AUTO_MULTIDECLARATIONS);
    PRINT_MACRO(BOOST_NO_AUTO_PTR);
-   PRINT_MACRO(BOOST_NO_CHAR16_T);
-   PRINT_MACRO(BOOST_NO_CHAR32_T);
    PRINT_MACRO(BOOST_NO_COMPLETE_VALUE_INITIALIZATION);
-   PRINT_MACRO(BOOST_NO_CONSTEXPR);
    PRINT_MACRO(BOOST_NO_CTYPE_FUNCTIONS);
    PRINT_MACRO(BOOST_NO_CV_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_CV_VOID_SPECIALIZATIONS);
@@ -996,6 +991,18 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_CWCTYPE);
    PRINT_MACRO(BOOST_NO_CXX11_ALLOCATOR);
    PRINT_MACRO(BOOST_NO_CXX11_ATOMIC_SMART_PTR);
+   PRINT_MACRO(BOOST_NO_CXX11_AUTO_DECLARATIONS);
+   PRINT_MACRO(BOOST_NO_CXX11_AUTO_MULTIDECLARATIONS);
+   PRINT_MACRO(BOOST_NO_CXX11_CHAR16_T);
+   PRINT_MACRO(BOOST_NO_CXX11_CHAR32_T);
+   PRINT_MACRO(BOOST_NO_CXX11_CONSTEXPR);
+   PRINT_MACRO(BOOST_NO_CXX11_DECLTYPE);
+   PRINT_MACRO(BOOST_NO_CXX11_DECLTYPE_N3276);
+   PRINT_MACRO(BOOST_NO_CXX11_DEFAULTED_FUNCTIONS);
+   PRINT_MACRO(BOOST_NO_CXX11_DELETED_FUNCTIONS);
+   PRINT_MACRO(BOOST_NO_CXX11_EXPLICIT_CONVERSION_OPERATORS);
+   PRINT_MACRO(BOOST_NO_CXX11_EXTERN_TEMPLATE);
+   PRINT_MACRO(BOOST_NO_CXX11_FUNCTION_TEMPLATE_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_ARRAY);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_CHRONO);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_CODECVT);
@@ -1015,20 +1022,27 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_CXX11_HDR_TYPE_TRAITS);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_UNORDERED_MAP);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_UNORDERED_SET);
+   PRINT_MACRO(BOOST_NO_CXX11_LAMBDAS);
+   PRINT_MACRO(BOOST_NO_CXX11_LOCAL_CLASS_TEMPLATE_PARAMETERS);
+   PRINT_MACRO(BOOST_NO_CXX11_NOEXCEPT);
+   PRINT_MACRO(BOOST_NO_CXX11_NULLPTR);
+   PRINT_MACRO(BOOST_NO_CXX11_RANGE_BASED_FOR);
+   PRINT_MACRO(BOOST_NO_CXX11_RAW_LITERALS);
+   PRINT_MACRO(BOOST_NO_CXX11_RVALUE_REFERENCES);
+   PRINT_MACRO(BOOST_NO_CXX11_SCOPED_ENUMS);
    PRINT_MACRO(BOOST_NO_CXX11_SMART_PTR);
-   PRINT_MACRO(BOOST_NO_DECLTYPE);
-   PRINT_MACRO(BOOST_NO_DECLTYPE_N3276);
-   PRINT_MACRO(BOOST_NO_DEFAULTED_FUNCTIONS);
-   PRINT_MACRO(BOOST_NO_DELETED_FUNCTIONS);
+   PRINT_MACRO(BOOST_NO_CXX11_STATIC_ASSERT);
+   PRINT_MACRO(BOOST_NO_CXX11_TEMPLATE_ALIASES);
+   PRINT_MACRO(BOOST_NO_CXX11_UNICODE_LITERALS);
+   PRINT_MACRO(BOOST_NO_CXX11_UNIFIED_INITIALIZATION_SYNTAX);
+   PRINT_MACRO(BOOST_NO_CXX11_VARIADIC_MACROS);
+   PRINT_MACRO(BOOST_NO_CXX11_VARIADIC_TEMPLATES);
    PRINT_MACRO(BOOST_NO_DEPENDENT_NESTED_DERIVATIONS);
    PRINT_MACRO(BOOST_NO_DEPENDENT_TYPES_IN_TEMPLATE_VALUE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_EXCEPTIONS);
    PRINT_MACRO(BOOST_NO_EXCEPTION_STD_NAMESPACE);
-   PRINT_MACRO(BOOST_NO_EXPLICIT_CONVERSION_OPERATORS);
    PRINT_MACRO(BOOST_NO_EXPLICIT_FUNCTION_TEMPLATE_ARGUMENTS);
-   PRINT_MACRO(BOOST_NO_EXTERN_TEMPLATE);
    PRINT_MACRO(BOOST_NO_FENV_H);
-   PRINT_MACRO(BOOST_NO_FUNCTION_TEMPLATE_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_FUNCTION_TEMPLATE_ORDERING);
    PRINT_MACRO(BOOST_NO_FUNCTION_TYPE_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_INCLASS_MEMBER_INITIALIZATION);
@@ -1037,10 +1051,8 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_IOSFWD);
    PRINT_MACRO(BOOST_NO_IOSTREAM);
    PRINT_MACRO(BOOST_NO_IS_ABSTRACT);
-   PRINT_MACRO(BOOST_NO_LAMBDAS);
    PRINT_MACRO(BOOST_NO_LIMITS);
    PRINT_MACRO(BOOST_NO_LIMITS_COMPILE_TIME_CONSTANTS);
-   PRINT_MACRO(BOOST_NO_LOCAL_CLASS_TEMPLATE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_LONG_LONG);
    PRINT_MACRO(BOOST_NO_LONG_LONG_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_MEMBER_FUNCTION_SPECIALIZATIONS);
@@ -1049,21 +1061,15 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_KEYWORD);
    PRINT_MACRO(BOOST_NO_MS_INT64_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_NESTED_FRIENDSHIP);
-   PRINT_MACRO(BOOST_NO_NOEXCEPT);
-   PRINT_MACRO(BOOST_NO_NULLPTR);
-   PRINT_MACRO(BOOST_NO_NUMERIC_LIMITS_LOWEST);
+   PRINT_MACRO(BOOST_NO_CXX11_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_OPERATORS_IN_NAMESPACE);
    PRINT_MACRO(BOOST_NO_PARTIAL_SPECIALIZATION_IMPLICIT_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_CONST);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_TEMPLATE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_PRIVATE_IN_AGGREGATE);
-   PRINT_MACRO(BOOST_NO_RAW_LITERALS);
    PRINT_MACRO(BOOST_NO_RTTI);
-   PRINT_MACRO(BOOST_NO_RVALUE_REFERENCES);
-   PRINT_MACRO(BOOST_NO_SCOPED_ENUMS);
    PRINT_MACRO(BOOST_NO_SFINAE);
    PRINT_MACRO(BOOST_NO_SFINAE_EXPR);
-   PRINT_MACRO(BOOST_NO_STATIC_ASSERT);
    PRINT_MACRO(BOOST_NO_STDC_NAMESPACE);
    PRINT_MACRO(BOOST_NO_STD_ALLOCATOR);
    PRINT_MACRO(BOOST_NO_STD_DISTANCE);
@@ -1082,19 +1088,14 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_SWPRINTF);
    PRINT_MACRO(BOOST_NO_TEMPLATED_IOSTREAMS);
    PRINT_MACRO(BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS);
-   PRINT_MACRO(BOOST_NO_TEMPLATE_ALIASES);
    PRINT_MACRO(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION);
    PRINT_MACRO(BOOST_NO_TEMPLATE_TEMPLATES);
    PRINT_MACRO(BOOST_NO_TWO_PHASE_NAME_LOOKUP);
    PRINT_MACRO(BOOST_NO_TYPEID);
    PRINT_MACRO(BOOST_NO_TYPENAME_WITH_CTOR);
-   PRINT_MACRO(BOOST_NO_UNICODE_LITERALS);
-   PRINT_MACRO(BOOST_NO_UNIFIED_INITIALIZATION_SYNTAX);
    PRINT_MACRO(BOOST_NO_UNREACHABLE_RETURN_DETECTION);
    PRINT_MACRO(BOOST_NO_USING_DECLARATION_OVERLOADS_FROM_TYPENAME_BASE);
    PRINT_MACRO(BOOST_NO_USING_TEMPLATE);
-   PRINT_MACRO(BOOST_NO_VARIADIC_MACROS);
-   PRINT_MACRO(BOOST_NO_VARIADIC_TEMPLATES);
    PRINT_MACRO(BOOST_NO_VOID_RETURNS);
 
 
@@ -1112,6 +1113,10 @@ void print_boost_macros()
 
 
 
+
+
+
+
 
    // END GENERATED BLOCK
 
