@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2006. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2006-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -118,7 +118,7 @@ bool test_allocation_shrink(Allocator &a)
 		 std::memset(buffers[i], 0, a.size(buffers[i]));
       }
    }
-  
+
    //Deallocate it in non sequential order
    for(int j = 0, max = (int)buffers.size()
       ;j < max
@@ -170,7 +170,7 @@ bool test_allocation_expand(Allocator &a)
          preferred_size = min_size*2;
       }
    }
-  
+
    //Deallocate it in non sequential order
    for(int j = 0, max = (int)buffers.size()
       ;j < max
@@ -243,7 +243,7 @@ bool test_allocation_shrink_and_expand(Allocator &a)
          return false;
       }
    }
-  
+
    //Deallocate it in non sequential order
    for(int j = 0, max = (int)buffers.size()
       ;j < max
@@ -310,7 +310,7 @@ bool test_allocation_deallocation_expand(Allocator &a)
          }
       }
    }
-  
+
    //Now erase null values from the vector
    buffers.erase( std::remove(buffers.begin(), buffers.end(), static_cast<void*>(0))
                 , buffers.end());
@@ -349,7 +349,7 @@ bool test_allocation_with_reuse(Allocator &a)
          std::memset(ptr, 0, size);
          buffers.push_back(ptr);
       }
-     
+
       //Now deallocate all except the latest
       //Now try to expand to the double of the sizeof_object
       for(int i = 0, max = (int)buffers.size() - 1
@@ -381,7 +381,7 @@ bool test_allocation_with_reuse(Allocator &a)
       }
       //There is only a single block so deallocate it
       a.deallocate(ptr);
-     
+
       if(!a.all_memory_deallocated() || !a.check_sanity())
          return false;
    }
@@ -405,7 +405,7 @@ bool test_aligned_allocation(Allocator &a)
                continue_loop = false;
             break;
          }
-        
+
          if(((std::size_t)ptr & (j - 1)) != 0)
             return false;
          a.deallocate(ptr);
@@ -441,7 +441,7 @@ bool test_continuous_aligned_allocation(Allocator &a)
             else{
                any_allocated = true;
             }
-           
+
             if(((std::size_t)ptr & (j - 1)) != 0)
                return false;
          }
@@ -679,8 +679,7 @@ bool test_many_equal_allocation(Allocator &a)
 
          typename multiallocation_chain::size_type n = chain.size();
          while(!chain.empty()){
-            buffers.push_back(ipcdetail::to_raw_pointer(chain.front()));
-            chain.pop_front();
+            buffers.push_back(ipcdetail::to_raw_pointer(chain.pop_front()));
          }
          if(n != std::size_t((i+1)*2))
             return false;
@@ -788,8 +787,7 @@ bool test_many_different_allocation(Allocator &a)
             break;
          typename multiallocation_chain::size_type n = chain.size();
          while(!chain.empty()){
-            buffers.push_back(ipcdetail::to_raw_pointer(chain.front()));
-            chain.pop_front();
+            buffers.push_back(ipcdetail::to_raw_pointer(chain.pop_front()));
          }
          if(n != ArraySize)
             return false;
