@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2004-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2004-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -33,7 +33,7 @@ static const char *name_initialization_routine()
 
 static const std::size_t ShmSize = 1000;
 typedef ipcdetail::managed_open_or_create_impl
-   <windows_shared_memory, false> windows_shared_memory_t;
+   <windows_shared_memory, 0, false> windows_shared_memory_t;
 
 //This wrapper is necessary to have a common constructor
 //in generic named_creation_template functions
