@@ -9,8 +9,8 @@
 
 #include <boost/test/unit_test.hpp>
 
-#include <libs/thread/test/util.inl>
-#include <libs/thread/test/condition_test_common.hpp>
+#include "./util.inl"
+#include "./condition_test_common.hpp"
 
 void do_test_condition_notify_one_wakes_from_wait()
 {
