@@ -1,24 +1,35 @@
-//  (C) Copyright 2008-9 Anthony Williams 
+//  (C) Copyright 2008-10 Anthony Williams
 //
 //  Distributed under the Boost Software License, Version 1.0. (See
 //  accompanying file LICENSE_1_0.txt or copy at
 //  http://www.boost.org/LICENSE_1_0.txt)
 
-#include "boost/thread/thread.hpp"
-#include "boost/thread/mutex.hpp"
-#include "boost/thread/condition.hpp"
-#include "boost/thread/future.hpp"
+#include <boost/thread/thread.hpp>
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/condition.hpp>
+#include <boost/thread/future.hpp>
 #include <utility>
 #include <memory>
 #include <string>
+#include <iostream>
 
 #include <boost/test/unit_test.hpp>
 
-#ifdef BOOST_HAS_RVALUE_REFS
+#define LOG \
+  if (false) {} else std::cout << std::endl << __FILE__ << "[" << __LINE__ << "]"
+
+#ifndef BOOST_NO_RVALUE_REFERENCES
     template<typename T>
     typename boost::remove_reference<T>::type&& cast_to_rval(T&& t)
     {
-        return t;
+        return static_cast<typename boost::remove_reference<T>::type&&>(t);
+    }
+#else
+#if defined BOOST_THREAD_USES_MOVE
+    template<typename T>
+    boost::rv<T>& cast_to_rval(T& t)
+    {
+        return boost::move(t);
     }
 #else
     template<typename T>
@@ -27,37 +38,22 @@
         return boost::move(t);
     }
 #endif
+#endif
 
 struct X
 {
-private:
-    
-    X(X& other);
-    
 public:
-    
     int i;
-    
+
+    BOOST_THREAD_MOVABLE_ONLY(X)
     X():
         i(42)
     {}
-#ifdef BOOST_HAS_RVALUE_REFS
-    X(X&& other):
-        i(other.i)
+    X(BOOST_THREAD_RV_REF(X) other):
+        i(BOOST_THREAD_RV(other).i)
     {
-        other.i=0;
+      BOOST_THREAD_RV(other).i=0;
     }
-#else
-    X(boost::detail::thread_move_t<X> other):
-        i(other->i)
-    {
-        other->i=0;
-    }
-    operator boost::detail::thread_move_t<X>()
-    {
-        return boost::detail::thread_move_t<X>(*this);
-    }
-#endif
     ~X()
     {}
 };
@@ -88,22 +84,39 @@ void set_promise_exception_thread(boost::promise<int>* p)
 
 void test_store_value_from_thread()
 {
+    LOG;
+    try {
     boost::promise<int> pi2;
-    boost::unique_future<int> fi2=pi2.get_future();
+    LOG;
+    boost::unique_future<int> fi2(BOOST_THREAD_MAKE_RV_REF(pi2.get_future()));
+    LOG;
     boost::thread(set_promise_thread,&pi2);
+    LOG;
     int j=fi2.get();
+    LOG;
     BOOST_CHECK(j==42);
+    LOG;
     BOOST_CHECK(fi2.is_ready());
+    LOG;
     BOOST_CHECK(fi2.has_value());
+    LOG;
     BOOST_CHECK(!fi2.has_exception());
+    LOG;
     BOOST_CHECK(fi2.get_state()==boost::future_state::ready);
+    LOG;
+    }
+    catch (...)
+    {
+      BOOST_CHECK(false&&"Exception thrown");
+    }
 }
 
 
 void test_store_exception()
 {
+  LOG;
     boost::promise<int> pi3;
-    boost::unique_future<int> fi3=pi3.get_future();
+    boost::unique_future<int> fi3(BOOST_THREAD_MAKE_RV_REF(pi3.get_future()));
     boost::thread(set_promise_exception_thread,&pi3);
     try
     {
@@ -114,7 +127,7 @@ void test_store_exception()
     {
         BOOST_CHECK(true);
     }
-    
+
     BOOST_CHECK(fi3.is_ready());
     BOOST_CHECK(!fi3.has_value());
     BOOST_CHECK(fi3.has_exception());
@@ -123,6 +136,7 @@ void test_store_exception()
 
 void test_initial_state()
 {
+  LOG;
     boost::unique_future<int> fi;
     BOOST_CHECK(!fi.is_ready());
     BOOST_CHECK(!fi.has_value());
@@ -132,6 +146,7 @@ void test_initial_state()
     try
     {
         i=fi.get();
+        (void)i;
         BOOST_CHECK(false);
     }
     catch(boost::future_uninitialized)
@@ -142,9 +157,10 @@ void test_initial_state()
 
 void test_waiting_future()
 {
+  LOG;
     boost::promise<int> pi;
     boost::unique_future<int> fi;
-    fi=pi.get_future();
+    fi=BOOST_THREAD_MAKE_RV_REF(pi.get_future());
 
     int i=0;
     BOOST_CHECK(!fi.is_ready());
@@ -156,8 +172,9 @@ void test_waiting_future()
 
 void test_cannot_get_future_twice()
 {
+  LOG;
     boost::promise<int> pi;
-    pi.get_future();
+    BOOST_THREAD_MAKE_RV_REF(pi.get_future());
 
     try
     {
@@ -172,12 +189,13 @@ void test_cannot_get_future_twice()
 
 void test_set_value_updates_future_state()
 {
+  LOG;
     boost::promise<int> pi;
     boost::unique_future<int> fi;
-    fi=pi.get_future();
+    fi=BOOST_THREAD_MAKE_RV_REF(pi.get_future());
 
     pi.set_value(42);
-    
+
     BOOST_CHECK(fi.is_ready());
     BOOST_CHECK(fi.has_value());
     BOOST_CHECK(!fi.has_exception());
@@ -186,12 +204,13 @@ void test_set_value_updates_future_state()
 
 void test_set_value_can_be_retrieved()
 {
+  LOG;
     boost::promise<int> pi;
     boost::unique_future<int> fi;
-    fi=pi.get_future();
+    fi=BOOST_THREAD_MAKE_RV_REF(pi.get_future());
 
     pi.set_value(42);
-    
+
     int i=0;
     BOOST_CHECK(i=fi.get());
     BOOST_CHECK(i==42);
@@ -203,12 +222,13 @@ void test_set_value_can_be_retrieved()
 
 void test_set_value_can_be_moved()
 {
+  LOG;
 //     boost::promise<int> pi;
 //     boost::unique_future<int> fi;
-//     fi=pi.get_future();
+//     fi=BOOST_THREAD_MAKE_RV_REF(pi.get_future());
 
 //     pi.set_value(42);
-    
+
 //     int i=0;
 //     BOOST_CHECK(i=fi.get());
 //     BOOST_CHECK(i==42);
@@ -220,8 +240,9 @@ void test_set_value_can_be_moved()
 
 void test_future_from_packaged_task_is_waiting()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     int i=0;
     BOOST_CHECK(!fi.is_ready());
     BOOST_CHECK(!fi.has_value());
@@ -232,8 +253,9 @@ void test_future_from_packaged_task_is_waiting()
 
 void test_invoking_a_packaged_task_populates_future()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     pt();
 
@@ -248,6 +270,7 @@ void test_invoking_a_packaged_task_populates_future()
 
 void test_invoking_a_packaged_task_twice_throws()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
 
     pt();
@@ -265,6 +288,7 @@ void test_invoking_a_packaged_task_twice_throws()
 
 void test_cannot_get_future_twice_from_task()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
     pt.get_future();
     try
@@ -280,8 +304,9 @@ void test_cannot_get_future_twice_from_task()
 
 void test_task_stores_exception_if_function_throws()
 {
+  LOG;
     boost::packaged_task<int> pt(throw_runtime_error);
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     pt();
 
@@ -302,13 +327,14 @@ void test_task_stores_exception_if_function_throws()
     {
         BOOST_CHECK(!"Unknown exception thrown");
     }
-    
+
 }
 
 void test_void_promise()
 {
+  LOG;
     boost::promise<void> p;
-    boost::unique_future<void> f=p.get_future();
+    boost::unique_future<void> f(BOOST_THREAD_MAKE_RV_REF(p.get_future()));
     p.set_value();
     BOOST_CHECK(f.is_ready());
     BOOST_CHECK(f.has_value());
@@ -319,8 +345,9 @@ void test_void_promise()
 
 void test_reference_promise()
 {
+  LOG;
     boost::promise<int&> p;
-    boost::unique_future<int&> f=p.get_future();
+    boost::unique_future<int&> f(BOOST_THREAD_MAKE_RV_REF(p.get_future()));
     int i=42;
     p.set_value(i);
     BOOST_CHECK(f.is_ready());
@@ -335,8 +362,9 @@ void do_nothing()
 
 void test_task_returning_void()
 {
+  LOG;
     boost::packaged_task<void> pt(do_nothing);
-    boost::unique_future<void> fi=pt.get_future();
+    boost::unique_future<void> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     pt();
 
@@ -355,8 +383,9 @@ int& return_ref()
 
 void test_task_returning_reference()
 {
+  LOG;
     boost::packaged_task<int&> pt(return_ref);
-    boost::unique_future<int&> fi=pt.get_future();
+    boost::unique_future<int&> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     pt();
 
@@ -370,6 +399,7 @@ void test_task_returning_reference()
 
 void test_shared_future()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
     boost::unique_future<int> fi=pt.get_future();
 
@@ -389,8 +419,9 @@ void test_shared_future()
 
 void test_copies_of_shared_future_become_ready_together()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     boost::shared_future<int> sf(::cast_to_rval(fi));
     boost::shared_future<int> sf2(sf);
@@ -427,8 +458,9 @@ void test_copies_of_shared_future_become_ready_together()
 
 void test_shared_future_can_be_move_assigned_from_unique_future()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     boost::shared_future<int> sf;
     sf=::cast_to_rval(fi);
@@ -442,8 +474,9 @@ void test_shared_future_can_be_move_assigned_from_unique_future()
 
 void test_shared_future_void()
 {
+  LOG;
     boost::packaged_task<void> pt(do_nothing);
-    boost::unique_future<void> fi=pt.get_future();
+    boost::unique_future<void> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     boost::shared_future<void> sf(::cast_to_rval(fi));
     BOOST_CHECK(fi.get_state()==boost::future_state::uninitialized);
@@ -459,8 +492,9 @@ void test_shared_future_void()
 
 void test_shared_future_ref()
 {
+  LOG;
     boost::promise<int&> p;
-    boost::shared_future<int&> f(p.get_future());
+    boost::shared_future<int&> f(BOOST_THREAD_MAKE_RV_REF(p.get_future()));
     int i=42;
     p.set_value(i);
     BOOST_CHECK(f.is_ready());
@@ -472,11 +506,12 @@ void test_shared_future_ref()
 
 void test_can_get_a_second_future_from_a_moved_promise()
 {
+  LOG;
     boost::promise<int> pi;
-    boost::unique_future<int> fi=pi.get_future();
-    
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pi.get_future()));
+
     boost::promise<int> pi2(::cast_to_rval(pi));
-    boost::unique_future<int> fi2=pi.get_future();
+    boost::unique_future<int> fi2(BOOST_THREAD_MAKE_RV_REF(pi.get_future()));
 
     pi2.set_value(3);
     BOOST_CHECK(fi.is_ready());
@@ -489,11 +524,12 @@ void test_can_get_a_second_future_from_a_moved_promise()
 
 void test_can_get_a_second_future_from_a_moved_void_promise()
 {
+  LOG;
     boost::promise<void> pi;
-    boost::unique_future<void> fi=pi.get_future();
-    
+    boost::unique_future<void> fi(BOOST_THREAD_MAKE_RV_REF(pi.get_future()));
+
     boost::promise<void> pi2(::cast_to_rval(pi));
-    boost::unique_future<void> fi2=pi.get_future();
+    boost::unique_future<void> fi2(BOOST_THREAD_MAKE_RV_REF(pi.get_future()));
 
     pi2.set_value();
     BOOST_CHECK(fi.is_ready());
@@ -504,8 +540,9 @@ void test_can_get_a_second_future_from_a_moved_void_promise()
 
 void test_unique_future_for_move_only_udt()
 {
+  LOG;
     boost::promise<X> pt;
-    boost::unique_future<X> fi=pt.get_future();
+    boost::unique_future<X> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     pt.set_value(X());
     X res(fi.get());
@@ -514,27 +551,28 @@ void test_unique_future_for_move_only_udt()
 
 void test_unique_future_for_string()
 {
+  LOG;
     boost::promise<std::string> pt;
-    boost::unique_future<std::string> fi=pt.get_future();
+    boost::unique_future<std::string> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     pt.set_value(std::string("hello"));
     std::string res(fi.get());
     BOOST_CHECK(res=="hello");
 
     boost::promise<std::string> pt2;
-    fi=pt2.get_future();
+    fi=BOOST_THREAD_MAKE_RV_REF(pt2.get_future());
 
     std::string const s="goodbye";
-    
+
     pt2.set_value(s);
     res=fi.get();
     BOOST_CHECK(res=="goodbye");
 
     boost::promise<std::string> pt3;
-    fi=pt3.get_future();
+    fi=BOOST_THREAD_MAKE_RV_REF(pt3.get_future());
 
     std::string s2="foo";
-    
+
     pt3.set_value(s2);
     res=fi.get();
     BOOST_CHECK(res=="foo");
@@ -556,7 +594,7 @@ void wait_callback(boost::promise<int>& pi)
     }
 }
 
-void do_nothing_callback(boost::promise<int>& pi)
+void do_nothing_callback(boost::promise<int>& /*pi*/)
 {
     boost::lock_guard<boost::mutex> lk(callback_mutex);
     ++callback_called;
@@ -564,9 +602,10 @@ void do_nothing_callback(boost::promise<int>& pi)
 
 void test_wait_callback()
 {
+  LOG;
     callback_called=0;
     boost::promise<int> pi;
-    boost::unique_future<int> fi=pi.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pi.get_future()));
     pi.set_wait_callback(wait_callback);
     fi.wait();
     BOOST_CHECK(callback_called);
@@ -578,9 +617,10 @@ void test_wait_callback()
 
 void test_wait_callback_with_timed_wait()
 {
+  LOG;
     callback_called=0;
     boost::promise<int> pi;
-    boost::unique_future<int> fi=pi.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pi.get_future()));
     pi.set_wait_callback(do_nothing_callback);
     bool success=fi.timed_wait(boost::posix_time::milliseconds(10));
     BOOST_CHECK(callback_called);
@@ -601,6 +641,7 @@ void test_wait_callback_with_timed_wait()
 
 void wait_callback_for_task(boost::packaged_task<int>& pt)
 {
+  LOG;
     boost::lock_guard<boost::mutex> lk(callback_mutex);
     ++callback_called;
     try
@@ -615,9 +656,10 @@ void wait_callback_for_task(boost::packaged_task<int>& pt)
 
 void test_wait_callback_for_packaged_task()
 {
+  LOG;
     callback_called=0;
     boost::packaged_task<int> pt(make_int);
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     pt.set_wait_callback(wait_callback_for_task);
     fi.wait();
     BOOST_CHECK(callback_called);
@@ -629,12 +671,13 @@ void test_wait_callback_for_packaged_task()
 
 void test_packaged_task_can_be_moved()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int);
 
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
 
     BOOST_CHECK(!fi.is_ready());
-    
+
     boost::packaged_task<int> pt2(::cast_to_rval(pt));
 
     BOOST_CHECK(!fi.is_ready());
@@ -650,17 +693,18 @@ void test_packaged_task_can_be_moved()
     BOOST_CHECK(!fi.is_ready());
 
     pt2();
-    
+
     BOOST_CHECK(fi.is_ready());
 }
 
 void test_destroying_a_promise_stores_broken_promise()
 {
+  LOG;
     boost::unique_future<int> f;
-    
+
     {
         boost::promise<int> p;
-        f=p.get_future();
+        f=BOOST_THREAD_MAKE_RV_REF(p.get_future());
     }
     BOOST_CHECK(f.is_ready());
     BOOST_CHECK(f.has_exception());
@@ -675,11 +719,12 @@ void test_destroying_a_promise_stores_broken_promise()
 
 void test_destroying_a_packaged_task_stores_broken_promise()
 {
+  LOG;
     boost::unique_future<int> f;
-    
+
     {
         boost::packaged_task<int> p(make_int);
-        f=p.get_future();
+        f=BOOST_THREAD_MAKE_RV_REF(p.get_future());
     }
     BOOST_CHECK(f.is_ready());
     BOOST_CHECK(f.has_exception());
@@ -700,15 +745,16 @@ int make_int_slowly()
 
 void test_wait_for_either_of_two_futures_1()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
-    
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
+
     boost::thread(::cast_to_rval(pt));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2);
-    
+
     BOOST_CHECK(future==0);
     BOOST_CHECK(f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -717,15 +763,16 @@ void test_wait_for_either_of_two_futures_1()
 
 void test_wait_for_either_of_two_futures_2()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
-    
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
+
     boost::thread(::cast_to_rval(pt2));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2);
-    
+
     BOOST_CHECK(future==1);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(f2.is_ready());
@@ -734,17 +781,18 @@ void test_wait_for_either_of_two_futures_2()
 
 void test_wait_for_either_of_three_futures_1()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
-    
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
+
     boost::thread(::cast_to_rval(pt));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3);
-    
+
     BOOST_CHECK(future==0);
     BOOST_CHECK(f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -754,17 +802,18 @@ void test_wait_for_either_of_three_futures_1()
 
 void test_wait_for_either_of_three_futures_2()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
-    
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
+
     boost::thread(::cast_to_rval(pt2));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3);
-    
+
     BOOST_CHECK(future==1);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(f2.is_ready());
@@ -774,17 +823,18 @@ void test_wait_for_either_of_three_futures_2()
 
 void test_wait_for_either_of_three_futures_3()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
-    
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
+
     boost::thread(::cast_to_rval(pt3));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3);
-    
+
     BOOST_CHECK(future==2);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -794,19 +844,20 @@ void test_wait_for_either_of_three_futures_3()
 
 void test_wait_for_either_of_four_futures_1()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
-    
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
+
     boost::thread(::cast_to_rval(pt));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4);
-    
+
     BOOST_CHECK(future==0);
     BOOST_CHECK(f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -817,19 +868,20 @@ void test_wait_for_either_of_four_futures_1()
 
 void test_wait_for_either_of_four_futures_2()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
-    
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
+
     boost::thread(::cast_to_rval(pt2));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4);
-    
+
     BOOST_CHECK(future==1);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(f2.is_ready());
@@ -840,19 +892,20 @@ void test_wait_for_either_of_four_futures_2()
 
 void test_wait_for_either_of_four_futures_3()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
-    
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
+
     boost::thread(::cast_to_rval(pt3));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4);
-    
+
     BOOST_CHECK(future==2);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -863,19 +916,20 @@ void test_wait_for_either_of_four_futures_3()
 
 void test_wait_for_either_of_four_futures_4()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
-    
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
+
     boost::thread(::cast_to_rval(pt4));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4);
-    
+
     BOOST_CHECK(future==3);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -886,21 +940,22 @@ void test_wait_for_either_of_four_futures_4()
 
 void test_wait_for_either_of_five_futures_1()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
     boost::packaged_task<int> pt5(make_int_slowly);
-    boost::unique_future<int> f5(pt5.get_future());
-    
+    boost::unique_future<int> f5(BOOST_THREAD_MAKE_RV_REF(pt5.get_future()));
+
     boost::thread(::cast_to_rval(pt));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4,f5);
-    
+
     BOOST_CHECK(future==0);
     BOOST_CHECK(f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -912,21 +967,22 @@ void test_wait_for_either_of_five_futures_1()
 
 void test_wait_for_either_of_five_futures_2()
 {
-    boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+  LOG;
+   boost::packaged_task<int> pt(make_int_slowly);
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
     boost::packaged_task<int> pt5(make_int_slowly);
-    boost::unique_future<int> f5(pt5.get_future());
-    
+    boost::unique_future<int> f5(BOOST_THREAD_MAKE_RV_REF(pt5.get_future()));
+
     boost::thread(::cast_to_rval(pt2));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4,f5);
-    
+
     BOOST_CHECK(future==1);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(f2.is_ready());
@@ -937,21 +993,22 @@ void test_wait_for_either_of_five_futures_2()
 }
 void test_wait_for_either_of_five_futures_3()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
     boost::packaged_task<int> pt5(make_int_slowly);
-    boost::unique_future<int> f5(pt5.get_future());
-    
+    boost::unique_future<int> f5(BOOST_THREAD_MAKE_RV_REF(pt5.get_future()));
+
     boost::thread(::cast_to_rval(pt3));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4,f5);
-    
+
     BOOST_CHECK(future==2);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -962,21 +1019,22 @@ void test_wait_for_either_of_five_futures_3()
 }
 void test_wait_for_either_of_five_futures_4()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
     boost::packaged_task<int> pt5(make_int_slowly);
-    boost::unique_future<int> f5(pt5.get_future());
-    
+    boost::unique_future<int> f5(BOOST_THREAD_MAKE_RV_REF(pt5.get_future()));
+
     boost::thread(::cast_to_rval(pt4));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4,f5);
-    
+
     BOOST_CHECK(future==3);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -987,21 +1045,22 @@ void test_wait_for_either_of_five_futures_4()
 }
 void test_wait_for_either_of_five_futures_5()
 {
+  LOG;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> f1(pt.get_future());
+    boost::unique_future<int> f1(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> f2(pt2.get_future());
+    boost::unique_future<int> f2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     boost::packaged_task<int> pt3(make_int_slowly);
-    boost::unique_future<int> f3(pt3.get_future());
+    boost::unique_future<int> f3(BOOST_THREAD_MAKE_RV_REF(pt3.get_future()));
     boost::packaged_task<int> pt4(make_int_slowly);
-    boost::unique_future<int> f4(pt4.get_future());
+    boost::unique_future<int> f4(BOOST_THREAD_MAKE_RV_REF(pt4.get_future()));
     boost::packaged_task<int> pt5(make_int_slowly);
-    boost::unique_future<int> f5(pt5.get_future());
-    
+    boost::unique_future<int> f5(BOOST_THREAD_MAKE_RV_REF(pt5.get_future()));
+
     boost::thread(::cast_to_rval(pt5));
-    
+
     unsigned const future=boost::wait_for_any(f1,f2,f3,f4,f5);
-    
+
     BOOST_CHECK(future==4);
     BOOST_CHECK(!f1.is_ready());
     BOOST_CHECK(!f2.is_ready());
@@ -1013,15 +1072,16 @@ void test_wait_for_either_of_five_futures_5()
 
 void test_wait_for_either_invokes_callbacks()
 {
+  LOG;
     callback_called=0;
     boost::packaged_task<int> pt(make_int_slowly);
-    boost::unique_future<int> fi=pt.get_future();
+    boost::unique_future<int> fi(BOOST_THREAD_MAKE_RV_REF(pt.get_future()));
     boost::packaged_task<int> pt2(make_int_slowly);
-    boost::unique_future<int> fi2=pt2.get_future();
+    boost::unique_future<int> fi2(BOOST_THREAD_MAKE_RV_REF(pt2.get_future()));
     pt.set_wait_callback(wait_callback_for_task);
 
     boost::thread(::cast_to_rval(pt));
-    
+
     boost::wait_for_any(fi,fi2);
     BOOST_CHECK(callback_called==1);
     BOOST_CHECK(fi.get()==42);
@@ -1029,6 +1089,7 @@ void test_wait_for_either_invokes_callbacks()
 
 void test_wait_for_any_from_range()
 {
+  LOG;
     unsigned const count=10;
     for(unsigned i=0;i<count;++i)
     {
@@ -1037,12 +1098,14 @@ void test_wait_for_any_from_range()
         for(unsigned j=0;j<count;++j)
         {
             tasks[j]=boost::packaged_task<int>(make_int_slowly);
-            futures[j]=tasks[j].get_future();
+            futures[j]=BOOST_THREAD_MAKE_RV_REF(tasks[j].get_future());
         }
         boost::thread(::cast_to_rval(tasks[i]));
-    
+
+        BOOST_CHECK(boost::wait_for_any(futures,futures)==futures);
+
         boost::unique_future<int>* const future=boost::wait_for_any(futures,futures+count);
-    
+
         BOOST_CHECK(future==(futures+i));
         for(unsigned j=0;j<count;++j)
         {
@@ -1061,17 +1124,18 @@ void test_wait_for_any_from_range()
 
 void test_wait_for_all_from_range()
 {
+  LOG;
     unsigned const count=10;
     boost::unique_future<int> futures[count];
     for(unsigned j=0;j<count;++j)
     {
         boost::packaged_task<int> task(make_int_slowly);
-        futures[j]=task.get_future();
+        futures[j]=BOOST_THREAD_MAKE_RV_REF(task.get_future());
         boost::thread(::cast_to_rval(task));
     }
-    
+
     boost::wait_for_all(futures,futures+count);
-    
+
     for(unsigned j=0;j<count;++j)
     {
         BOOST_CHECK(futures[j].is_ready());
@@ -1080,17 +1144,18 @@ void test_wait_for_all_from_range()
 
 void test_wait_for_all_two_futures()
 {
+  LOG;
     unsigned const count=2;
     boost::unique_future<int> futures[count];
     for(unsigned j=0;j<count;++j)
     {
         boost::packaged_task<int> task(make_int_slowly);
-        futures[j]=task.get_future();
+        futures[j]=BOOST_THREAD_MAKE_RV_REF(task.get_future());
         boost::thread(::cast_to_rval(task));
     }
-    
+
     boost::wait_for_all(futures[0],futures[1]);
-    
+
     for(unsigned j=0;j<count;++j)
     {
         BOOST_CHECK(futures[j].is_ready());
@@ -1099,17 +1164,18 @@ void test_wait_for_all_two_futures()
 
 void test_wait_for_all_three_futures()
 {
+  LOG;
     unsigned const count=3;
     boost::unique_future<int> futures[count];
     for(unsigned j=0;j<count;++j)
     {
         boost::packaged_task<int> task(make_int_slowly);
-        futures[j]=task.get_future();
+        futures[j]=BOOST_THREAD_MAKE_RV_REF(task.get_future());
         boost::thread(::cast_to_rval(task));
     }
-    
+
     boost::wait_for_all(futures[0],futures[1],futures[2]);
-    
+
     for(unsigned j=0;j<count;++j)
     {
         BOOST_CHECK(futures[j].is_ready());
@@ -1118,17 +1184,18 @@ void test_wait_for_all_three_futures()
 
 void test_wait_for_all_four_futures()
 {
+  LOG;
     unsigned const count=4;
     boost::unique_future<int> futures[count];
     for(unsigned j=0;j<count;++j)
     {
         boost::packaged_task<int> task(make_int_slowly);
-        futures[j]=task.get_future();
+        futures[j]=BOOST_THREAD_MAKE_RV_REF(task.get_future());
         boost::thread(::cast_to_rval(task));
     }
-    
+
     boost::wait_for_all(futures[0],futures[1],futures[2],futures[3]);
-    
+
     for(unsigned j=0;j<count;++j)
     {
         BOOST_CHECK(futures[j].is_ready());
@@ -1137,17 +1204,18 @@ void test_wait_for_all_four_futures()
 
 void test_wait_for_all_five_futures()
 {
+  LOG;
     unsigned const count=5;
     boost::unique_future<int> futures[count];
     for(unsigned j=0;j<count;++j)
     {
         boost::packaged_task<int> task(make_int_slowly);
-        futures[j]=task.get_future();
+        futures[j]=BOOST_THREAD_MAKE_RV_REF(task.get_future());
         boost::thread(::cast_to_rval(task));
     }
-    
+
     boost::wait_for_all(futures[0],futures[1],futures[2],futures[3],futures[4]);
-    
+
     for(unsigned j=0;j<count;++j)
     {
         BOOST_CHECK(futures[j].is_ready());
@@ -1155,9 +1223,9 @@ void test_wait_for_all_five_futures()
 }
 
 
-boost::unit_test_framework::test_suite* init_unit_test_suite(int, char*[])
+boost::unit_test::test_suite* init_unit_test_suite(int, char*[])
 {
-    boost::unit_test_framework::test_suite* test =
+    boost::unit_test::test_suite* test =
         BOOST_TEST_SUITE("Boost.Threads: futures test suite");
 
     test->add(BOOST_TEST_CASE(test_initial_state));
@@ -1216,3 +1284,16 @@ boost::unit_test_framework::test_suite* init_unit_test_suite(int, char*[])
 
     return test;
 }
+
+void remove_unused_warning()
+{
+
+  //../../../boost/test/results_collector.hpp:40:13: warning: unused function 'first_failed_assertion' [-Wunused-function]
+  //(void)first_failed_assertion;
+
+  //../../../boost/test/tools/floating_point_comparison.hpp:304:25: warning: unused variable 'check_is_close' [-Wunused-variable]
+  //../../../boost/test/tools/floating_point_comparison.hpp:326:25: warning: unused variable 'check_is_small' [-Wunused-variable]
+  (void)boost::test_tools::check_is_close;
+  (void)boost::test_tools::check_is_small;
+
+}
