@@ -67,14 +67,4 @@ int main()
   }
 }
 
-void remove_unused_warning()
-{
-  //../../../boost/system/error_code.hpp:214:36: warning: boost::system::posix_category defined but not used [-Wunused-variable]
-  //../../../boost/system/error_code.hpp:215:36: warning: boost::system::errno_ecat defined but not used [-Wunused-variable]
-  //../../../boost/system/error_code.hpp:216:36: warning: boost::system::native_ecat defined but not used [-Wunused-variable]
-
-  (void)boost::system::posix_category;
-  (void)boost::system::errno_ecat;
-  (void)boost::system::native_ecat;
-
-}
+#include "../../../remove_error_code_unused_warning.hpp"
